module opbinterface(
	OPB_clk, 
	OPB_select, 
	OPB_RNW, 
	OPB_hwXfer, 
	OPB_fwXfer, 
	OPB_seqAddr, 
	OPB_ABus, 
	OPB_DBus,
	Sln_xferAck, 
	Sln_hwAck, 
	Sln_fwAck, 
	Sln_errAck, 
	Sln_toutSup, 
	Sln_retry, 
	Sln_DBusEn, 
	Sln_DBus
	       );
   
   parameter C_BASEADDR = 32'hffffff00;
   parameter C_HIGHADDR = 32'hffffffff;
   parameter OPB_WIDTH = 32;

   //Status Byte Constants
   parameter RESET_BIT = 0;
   parameter MODE_BIT = 1;
   parameter GO_BIT = 2;
   parameter DATA_WE_BIT = 3;
   parameter FIFO_FULL_BIT = 4;
   parameter FIFO_EMPTY_BIT = 5;
   parameter DEVICE_RDY = 6;
   parameter PIPELINE_STOPPED_BIT = 7;

   
   input     OPB_clk;
   input     OPB_select;
   input     OPB_RNW;
   input     OPB_hwXfer;
   input     OPB_fwXfer;
   input     OPB_seqAddr;
   input [OPB_WIDTH - 1:0] OPB_ABus;
   input [OPB_WIDTH - 1:0] OPB_DBus;
   output 		   Sln_xferAck;
   output 		   Sln_hwAck;
   output 		   Sln_fwAck;
   output 		   Sln_errAck;
   output 		   Sln_toutSup;
   output 		   Sln_retry;
   output 		   Sln_DBusEn;
   output [OPB_WIDTH - 1:0] Sln_DBus;

   wire 		    Sln_xferAck;
   wire 		    Sln_hwAck;
   wire 		    Sln_fwAck;
   wire 		    Sln_errAck;
   wire 		    Sln_toutSup;
   wire 		    Sln_retry;
   wire 		    Sln_DBusEn;
   reg [OPB_WIDTH - 1:0]    Sln_DBus;
   
   reg [OPB_WIDTH - 1:0]    Status; //Base Address Containts Mode, Ready bit,reset
   //Parts that fill the data argument
   reg [OPB_WIDTH - 1:0]    k_i;
   reg [OPB_WIDTH - 1:0]    c2;
   reg [OPB_WIDTH - 1:0]    c1;
   reg [OPB_WIDTH - 1:0]    theta2;
   reg [OPB_WIDTH - 1:0]    theta1;
   reg [OPB_WIDTH - 1:0]    r_j0; //To wide for the OPB split in two
   reg [OPB_WIDTH - 1:0]    r_j1;
   //mem input
   reg [OPB_WIDTH - 1:0]    mem_ds_x;
   reg [OPB_WIDTH - 1:0]    mem_ds_y;
   reg [OPB_WIDTH - 1:0]    mem_ds_z;

   //Output
   wire [OPB_WIDTH - 1:0]   pout_x_0;
   wire [OPB_WIDTH - 1:0]   pout_x_1;
   wire [OPB_WIDTH - 1:0]   pout_y_0;
   wire [OPB_WIDTH - 1:0]   pout_y_1;
   wire [OPB_WIDTH - 1:0]   pout_z_0;
   wire [OPB_WIDTH - 1:0]   pout_z_1;

   //Device interaction wires
   wire 		    reset;
   wire 		    mode;
   wire 		    data_in_rdy;
   wire 		    go;
   wire 		    fifofull;
   wire 		    fifoempty;
   wire 		    deviceRdy;
   wire 		    pipelinestopped;

   wire [161:0] 	    data;
   wire [20:0] 		    mem_ds_x_int;
   wire [20:0] 		    mem_ds_y_int;
   wire [20:0] 		    mem_ds_z_int;
   wire [36:0] 		    pout_x_int;
   wire [36:0] 		    pout_y_int;
   wire [36:0] 		    pout_z_int;
   
   reg [1:0] 		    state; //0 -> Waiting for Valid Address 1-> Doing Read 2->Doing Write 3-> Error

   wire 		    ValidAddr; 		    


   initial
     begin
	state <= 0;
	Status <= 0;
     end
   
   assign 		    ValidAddr = (OPB_ABus <= C_HIGHADDR) & (OPB_ABus >= C_BASEADDR);
   
   assign 		    Sln_retry = state === 3;
   assign 		    Sln_errAck = state === 3;
   assign 		    Sln_DBusEn = state === 1;
   assign 		    Sln_xferAck = state === 1 | state === 2;
   assign 		    Sln_fwAck = state === 1 | state === 2;

   assign 		    Sln_hwAck = 0;
   assign 		    Sln_toutSup = 0;

   always @ (posedge OPB_clk)
     begin
	if(state === 0)
	  begin
	     if(OPB_select & ValidAddr)
	       begin
		  if(OPB_hwXfer & OPB_fwXfer)
		    begin
		       if(OPB_RNW)
			 begin
			    state <= 1;
			    case (OPB_ABus)
			      C_BASEADDR :
				Sln_DBus <= {24'b0,pipelinestopped,deviceRdy,fifoempty,fifofull,Status[3:0]};
			      C_BASEADDR + 4 :
				Sln_DBus <= pout_x_0;
			      C_BASEADDR + 8 :
				Sln_DBus <= pout_x_1;
			      C_BASEADDR + 12:
				Sln_DBus <= pout_y_0;
			      C_BASEADDR + 16:
				Sln_DBus <= pout_y_1;
			      C_BASEADDR + 20:
				Sln_DBus <= pout_z_0;
			      C_BASEADDR + 24:
				Sln_DBus <= pout_z_1;
			      default:
				Sln_DBus <= 0;
			    endcase
			 end // if (OPB_RNW)
		       else
			 begin
			    state <= 2;
			    Sln_DBus <= 0;
			    case (OPB_ABus)
			      C_BASEADDR     : //Status byte
				Status <= OPB_DBus;			
			      C_BASEADDR + 4 :
				k_i <= OPB_DBus;			
			      C_BASEADDR + 8 :
				c2 <= OPB_DBus;			
			      C_BASEADDR + 12:
				c1 <= OPB_DBus;			
			      C_BASEADDR + 16:
				theta2 <= OPB_DBus;			
			      C_BASEADDR + 20:
				theta1 <= OPB_DBus;			
			      C_BASEADDR + 24:
				r_j0 <= OPB_DBus;			
			      C_BASEADDR + 28:
				r_j1 <= OPB_DBus;			
			      C_BASEADDR + 32:
				mem_ds_x <= OPB_DBus;			
			      C_BASEADDR + 36:
				mem_ds_y <= OPB_DBus;			
			      C_BASEADDR + 40:
				mem_ds_z <= OPB_DBus;
			    endcase		      
			 end // else: !if(OPB_RNW)
		    end // if (OPB_hwXfer & OPB_fwXfer)
		  else
		    begin
		       state <= 3;  //Error
		       Sln_DBus <= 0;
		    end // else: !if(OPB_hwXfer & OPB_fwXfer)
	       end // if (OPB_select & ValidAddr)
	  end // if (state === 0)
	else if(state === 1 | state === 2 | state === 3)
	  begin
	     state <= 0; //After error go back to waiting
	     //Keep Bits high for only 1 cycle.
	     Status[RESET_BIT]<= 0;
	     Status[DATA_WE_BIT]<= 0;
	     Sln_DBus <= 0;
	  end
     end // always @ (posedge OPB_clk)

   
   //Device interaction
   assign reset = Status[RESET_BIT];
   assign mode = Status[MODE_BIT];
   assign go = Status[GO_BIT];
   assign data_in_rdy = Status[DATA_WE_BIT];
   assign data = {r_j1[30:0],r_j0[31:0],theta1[20:0],theta2[20:0],c1[17:0],c2[17:0],k_i[20:0]};
   assign mem_ds_x_int = mem_ds_x[20:0];
   assign mem_ds_y_int = mem_ds_y[20:0];
   assign mem_ds_z_int = mem_ds_z[20:0];
   assign pout_x_0 = pout_x_int[31:0];
   assign pout_x_1 = {27'b0,pout_x_int[36:32]};
   assign pout_y_0 = pout_y_int[31:0];
   assign pout_y_1 = {27'b0,pout_y_int[36:32]};
   assign pout_z_0 = pout_z_int[31:0];
   assign pout_z_1 = {27'b0,pout_z_int[36:32]};

   Controller WINE(.extclk(OPB_clk), .reset(reset), .mode(mode),
  		   .mem_x(mem_ds_x_int), .mem_y(mem_ds_y_int), .mem_z(mem_ds_z_int),
  		   .go(go), .data_in(data), .data_in_rdy(data_in_rdy),
  		   .fifofull(fifofull), .fifoempty(fifoempty), .deviceRdy(deviceRdy),
  		   .pout_x(pout_x_int), .pout_y(pout_y_int), .pout_z(pout_z_int),
  		   .pipelinestopped(pipelinestopped));
   
endmodule









