library IEEE;
use IEEE.STD_Logic_1164.all; 
use IEEE.STD_Logic_arith.all;
use IEEE.STD_Logic_signed.all;

entity enable_cntr is
  generic(AWIDTH	: INTEGER := 32;
	  INSTR_LOWERBND : STD_LOGIC_VECTOR(0 TO 31) := X"0080_0100";
	  INSTR_UPPERBND : STD_LOGIC_VECTOR(0 TO 31) := X"0080_01f4");
  port( PC_EX		:	in	STD_LOGIC_VECTOR(0 TO AWIDTH-1);
	valid_instr	:	in	STD_LOGIC;
	cntr_enable	:	out	STD_LOGIC);

end enable_cntr;

architecture behaviour of enable_cntr is
  signal met_lowerbnd:	STD_LOGIC;
  signal met_upperbnd:	STD_LOGIC;
  signal base_sub:	STD_LOGIC_VECTOR(0 to AWIDTH-3);
  signal high_sub:	STD_LOGIC_VECTOR(0 to AWIDTH-3);
  signal in_range:	STD_LOGIC;

begin

	--in range = equal or greater than
	base_sub <= PC_EX(0 TO AWIDTH-3) - INSTR_LOWERBND(0 TO AWIDTH-3); 
	high_sub <= INSTR_UPPERBND(0 TO AWIDTH-3) - PC_EX(0 TO AWIDTH-3); 

	met_lowerbnd <= base_sub(0); --+ve means MSB is 0
	met_upperbnd <= high_sub(0); --+ve means MSB is 0

	in_range <= met_lowerbnd nor met_upperbnd; --if either is 1, disable
	cntr_enable <= in_range and valid_instr;

end behaviour;
